@@ -66,6 +66,8 @@ class test_create_torrent(unittest.TestCase):
         ct.add_tracker('bar')
         ct.set_root_cert('1234567890')
         ct.add_collection('1337')
+        for i in range(ct.num_pieces()):
+            ct.set_hash(i, b'abababababababababab')
         entry = ct.generate()
         encoded = lt.bencode(entry)
         print(encoded)
@@ -75,7 +77,7 @@ class test_create_torrent(unittest.TestCase):
         encoded[1] = b'0e' + encoded[1].split(b'e', 1)[1]
         encoded = b'13:creation datei'.join(encoded)
 
-        self.assertEqual(encoded, b'd8:announce3:bar13:creation datei0e9:httpseeds3:bar4:infod11:collectionsl4:1337e5:filesld6:lengthi2000e4:pathl5:file2eed6:lengthi1000e4:pathl5:file1eee4:name4:test12:piece lengthi16384e6:pieces20:\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x008:ssl-cert10:1234567890e8:url-list3:fooe')
+        self.assertEqual(encoded, b'd8:announce3:bar13:creation datei0e9:httpseeds3:bar4:infod11:collectionsl4:1337e5:filesld6:lengthi1000e4:pathl5:file1eed4:attr1:p6:lengthi15384e4:pathl4:.pad5:15384eed6:lengthi2000e4:pathl5:file2eee4:name4:test12:piece lengthi16384e6:pieces40:abababababababababababababababababababab8:ssl-cert10:1234567890e8:url-list3:fooe')
 
 
 class test_session_stats(unittest.TestCase):
@@ -121,7 +123,7 @@ class test_torrent_handle(unittest.TestCase):
         self.ses = lt.session(settings)
         self.ti = lt.torrent_info('url_seed_multi.torrent')
         with self.assertRaises(RuntimeError):
-            self.ses.add_torrent({'ti': self.ti, 'save_path': os.getcwd(), 'info_hash': b'abababababababababab'})
+            self.ses.add_torrent({'ti': self.ti, 'save_path': os.getcwd(), 'info_hashes': b'abababababababababab'})
 
     def test_move_storage(self):
         self.setup()
@@ -221,11 +223,11 @@ class test_torrent_handle(unittest.TestCase):
         self.assertEqual(trackers[0]['url'], 'udp://tracker1.com')
         # this is not necessarily 0, it could also be (EHOSTUNREACH) if the
         # local machine doesn't support the address family
-        expect_value = trackers[0]['endpoints'][0]['last_error']['value']
+        expect_value = trackers[0]['endpoints'][0]['info_hashes'][0]['last_error']['value']
         pickled_trackers = pickle.dumps(trackers)
         unpickled_trackers = pickle.loads(pickled_trackers)
         self.assertEqual(unpickled_trackers[0]['url'], 'udp://tracker1.com')
-        self.assertEqual(unpickled_trackers[0]['endpoints'][0]['last_error']['value'], expect_value)
+        self.assertEqual(unpickled_trackers[0]['endpoints'][0]['info_hashes'][0]['last_error']['value'], expect_value)
 
     def test_file_status(self):
         self.setup()
@@ -265,7 +267,7 @@ class test_torrent_handle(unittest.TestCase):
         self.setup()
         st = self.h.status()
         ti = st.handle
-        self.assertEqual(ti.info_hash(), self.ti.info_hash())
+        self.assertEqual(ti.info_hashes(), self.ti.info_hashes())
         # make sure we can compare torrent_status objects
         st2 = self.h.status()
         self.assertEqual(st2, st)
@@ -283,7 +285,7 @@ class test_torrent_handle(unittest.TestCase):
         tp = lt.read_resume_data(resume_data)
 
         self.assertEqual(tp.name, 'test')
-        self.assertEqual(tp.info_hash, lt.sha1_hash('abababababababababab'))
+        self.assertEqual(tp.info_hashes.v1, lt.sha1_hash('abababababababababab'))
         self.assertEqual(tp.file_priorities, [0, 1, 1])
         self.assertEqual(tp.peers, [('1.1.1.1', 1), ('2.2.2.2', 2)])
 
@@ -343,7 +345,6 @@ class test_torrent_handle(unittest.TestCase):
         # piece priorities weren't set explicitly, but they were updated by the
         # file priorities being set
         self.assertEqual(self.h.get_piece_priorities(), [1])
-        self.assertEqual(self.ti.merkle_tree(), [])
         self.assertEqual(self.st.verified_pieces, [])
 
 
@@ -390,6 +391,16 @@ class TestAddPiece(unittest.TestCase):
 
 class test_torrent_info(unittest.TestCase):
 
+    def test_non_ascii_file(self):
+        try:
+            shutil.copy('base.torrent', 'base-\u745E\u5177.torrent')
+        except shutil.SameFileError:
+            pass
+        ti = lt.torrent_info('base-\u745E\u5177.torrent')
+
+        self.assertTrue(len(ti.info_section()) != 0)
+        self.assertTrue(len(ti.hash_for_piece(0)) != 0)
+
     def test_bencoded_constructor(self):
         # things that can be converted to a bencoded entry, will be interpreted
         # as such and encoded
@@ -444,10 +455,10 @@ class test_torrent_info(unittest.TestCase):
                 'piece length': 16 * 1024,
                 'pieces': 'aaaaaaaaaaaaaaaaaaaa'}}, {'max_buffer_size': 1}))
 
-    def test_metadata(self):
+    def test_info_section(self):
         ti = lt.torrent_info('base.torrent')
 
-        self.assertTrue(len(ti.metadata()) != 0)
+        self.assertTrue(len(ti.info_section()) != 0)
         self.assertTrue(len(ti.hash_for_piece(0)) != 0)
 
     def test_torrent_info_bytes_overload(self):
@@ -476,12 +487,21 @@ class test_torrent_info(unittest.TestCase):
         self.assertEqual(ae.verified, False)
         self.assertEqual(ae.source, 0)
 
-    def test_torrent_info_sha1_hash_overload(self):
-        ti = lt.torrent_info(lt.sha1_hash('a' * 20))
-        self.assertEqual(ti.info_hash(), lt.sha1_hash('a' * 20))
+    def test_torrent_info_sha1_overload(self):
+        ti = lt.torrent_info(lt.info_hash_t(lt.sha1_hash(b'a' * 20)))
+        self.assertEqual(ti.info_hash(), lt.sha1_hash(b'a' * 20))
+        self.assertEqual(ti.info_hashes().v1, lt.sha1_hash(b'a' * 20))
 
         ti_copy = lt.torrent_info(ti)
-        self.assertEqual(ti_copy.info_hash(), lt.sha1_hash('a' * 20))
+        self.assertEqual(ti_copy.info_hash(), lt.sha1_hash(b'a' * 20))
+        self.assertEqual(ti_copy.info_hashes().v1, lt.sha1_hash(b'a' * 20))
+
+    def test_torrent_info_sha256_overload(self):
+        ti = lt.torrent_info(lt.info_hash_t(lt.sha256_hash(b'a' * 32)))
+        self.assertEqual(ti.info_hashes().v2, lt.sha256_hash(b'a' * 32))
+
+        ti_copy = lt.torrent_info(ti)
+        self.assertEqual(ti_copy.info_hashes().v2, lt.sha256_hash(b'a' * 32))
 
     def test_url_seed(self):
         ti = lt.torrent_info('base.torrent')
@@ -546,7 +566,7 @@ class test_alerts(unittest.TestCase):
         print(st.progress)
         print(st.num_pieces)
         print(st.distributed_copies)
-        print(st.info_hash)
+        print(st.info_hashes)
         print(st.seeding_duration)
         print(st.last_upload)
         print(st.last_download)
@@ -647,6 +667,48 @@ class test_sha1hash(unittest.TestCase):
         s = lt.sha1_hash(binascii.unhexlify(h))
         self.assertEqual(h, str(s))
 
+    def test_hash(self):
+        self.assertNotEqual(hash(lt.sha1_hash(b'b' * 20)), hash(lt.sha1_hash(b'a' * 20)))
+        self.assertEqual(hash(lt.sha1_hash(b'b' * 20)), hash(lt.sha1_hash(b'b' * 20)))
+
+class test_sha256hash(unittest.TestCase):
+
+    def test_sha1hash(self):
+        h = 'a0' * 32
+        s = lt.sha256_hash(binascii.unhexlify(h))
+        self.assertEqual(h, str(s))
+
+    def test_hash(self):
+        self.assertNotEqual(hash(lt.sha256_hash(b'b' * 32)), hash(lt.sha256_hash(b'a' * 32)))
+        self.assertEqual(hash(lt.sha256_hash(b'b' * 32)), hash(lt.sha256_hash(b'b' * 32)))
+
+class test_info_hash(unittest.TestCase):
+
+    def test_info_hash(self):
+        s1 = lt.sha1_hash(b'a' * 20)
+        s2 = lt.sha256_hash(b'b' * 32)
+
+        ih1 = lt.info_hash_t(s1);
+        self.assertTrue(ih1.has_v1())
+        self.assertFalse(ih1.has_v2())
+        self.assertEqual(ih1.v1, s1)
+
+        ih2 = lt.info_hash_t(s2);
+        self.assertFalse(ih2.has_v1())
+        self.assertTrue(ih2.has_v2())
+        self.assertEqual(ih2.v2, s2)
+
+        ih12 = lt.info_hash_t(s1, s2);
+        self.assertTrue(ih12.has_v1())
+        self.assertTrue(ih12.has_v2())
+        self.assertEqual(ih12.v1, s1)
+        self.assertEqual(ih12.v2, s2)
+
+        self.assertNotEqual(hash(ih1), hash(ih2))
+        self.assertNotEqual(hash(ih1), hash(ih12))
+        self.assertEqual(hash(ih1), hash(lt.info_hash_t(s1)))
+        self.assertEqual(hash(ih2), hash(lt.info_hash_t(s2)))
+        self.assertEqual(hash(ih12), hash(lt.info_hash_t(s1, s2)))
 
 class test_magnet_link(unittest.TestCase):
 
@@ -654,19 +716,37 @@ class test_magnet_link(unittest.TestCase):
         ses = lt.session({})
         magnet = 'magnet:?xt=urn:btih:C6EIF4CCYDBTIJVG3APAGM7M4NDONCTI'
         p = lt.parse_magnet_uri(magnet)
-        self.assertEqual(str(p.info_hash), '178882f042c0c33426a6d81e0333ece346e68a68')
+        self.assertEqual(str(p.info_hashes.v1), '178882f042c0c33426a6d81e0333ece346e68a68')
         p.save_path = '.'
         h = ses.add_torrent(p)
         self.assertEqual(str(h.info_hash()), '178882f042c0c33426a6d81e0333ece346e68a68')
+        self.assertEqual(str(h.info_hashes().v1), '178882f042c0c33426a6d81e0333ece346e68a68')
 
     def test_parse_magnet_uri_dict(self):
         ses = lt.session({})
         magnet = 'magnet:?xt=urn:btih:C6EIF4CCYDBTIJVG3APAGM7M4NDONCTI'
         p = lt.parse_magnet_uri_dict(magnet)
-        self.assertEqual(binascii.hexlify(p['info_hash']), b'178882f042c0c33426a6d81e0333ece346e68a68')
+        self.assertEqual(binascii.hexlify(p['info_hashes']), b'178882f042c0c33426a6d81e0333ece346e68a68')
         p['save_path'] = '.'
         h = ses.add_torrent(p)
         self.assertEqual(str(h.info_hash()), '178882f042c0c33426a6d81e0333ece346e68a68')
+        self.assertEqual(str(h.info_hashes().v1), '178882f042c0c33426a6d81e0333ece346e68a68')
+
+    def test_add_deprecated_magnet_link(self):
+        ses = lt.session()
+        atp = lt.add_torrent_params()
+        atp.info_hashes = lt.info_hash_t(lt.sha1_hash(b"a" * 20))
+        h = ses.add_torrent(atp)
+
+        self.assertTrue(h.status().info_hashes == lt.info_hash_t(lt.sha1_hash(b"a" * 20)))
+
+    def test_add_magnet_link(self):
+        ses = lt.session()
+        atp = lt.add_torrent_params()
+        atp.info_hash = lt.sha1_hash(b"a" * 20)
+        h = ses.add_torrent(atp)
+
+        self.assertTrue(h.status().info_hashes == lt.info_hash_t(lt.sha1_hash(b"a" * 20)))
 
 
 class test_peer_class(unittest.TestCase):
@@ -744,9 +824,48 @@ class test_session(unittest.TestCase):
         sett = s.get_settings()
         self.assertEqual(sett['alert_mask'] & 0x7fffffff, 0x7fffffff)
 
+    def test_session_params(self):
+        sp = lt.session_params()
+        sp.settings = { 'alert_mask': lt.alert.category_t.all_categories }
+        s = lt.session(sp)
+        sett = s.get_settings()
+        self.assertEqual(sett['alert_mask'] & 0x7fffffff, 0x7fffffff)
+
+    def test_session_params_constructor(self):
+        sp = lt.session_params({ 'alert_mask': lt.alert.category_t.all_categories })
+        s = lt.session(sp)
+        sett = s.get_settings()
+        self.assertEqual(sett['alert_mask'] & 0x7fffffff, 0x7fffffff)
+
+    def test_session_params_ip_filter(self):
+        sp = lt.session_params()
+        sp.ip_filter.add_rule("1.1.1.1", "1.1.1.2", 1337)
+        self.assertEqual(sp.ip_filter.access("1.1.1.1"), 1337)
+        self.assertEqual(sp.ip_filter.access("1.1.1.2"), 1337)
+        self.assertEqual(sp.ip_filter.access("1.1.1.3"), 0)
+
+    def test_session_params_roundtrip_buf(self):
+
+        sp = lt.session_params()
+        sp.settings = { 'alert_mask': lt.alert.category_t.all_categories }
+
+        buf = lt.write_session_params_buf(sp)
+        sp2 = lt.read_session_params(buf)
+        self.assertEqual(sp2.settings['alert_mask'] & 0x7fffffff, 0x7fffffff)
+
+    def test_session_params_roundtrip_entry(self):
+
+        sp = lt.session_params()
+        sp.settings = { 'alert_mask': lt.alert.category_t.all_categories }
+
+        ent = lt.write_session_params(sp)
+        print(ent)
+        sp2 = lt.read_session_params(ent)
+        self.assertEqual(sp2.settings['alert_mask'] & 0x7fffffff, 0x7fffffff)
+
     def test_add_torrent(self):
         s = lt.session(settings)
-        s.add_torrent({'ti': lt.torrent_info('base.torrent'),
+        h = s.add_torrent({'ti': lt.torrent_info('base.torrent'),
                        'save_path': '.',
                        'dht_nodes': [('1.2.3.4', 6881), ('4.3.2.1', 6881)],
                        'http_seeds': ['http://test.com/seed'],
@@ -754,6 +873,37 @@ class test_session(unittest.TestCase):
                        'banned_peers': [('8.7.6.5', 6881)],
                        'file_priorities': [1, 1, 1, 2, 0]})
 
+    def test_find_torrent(self):
+        s = lt.session(settings)
+        h = s.add_torrent({'info_hash': b"a" * 20,
+                           'save_path': '.'})
+        self.assertTrue(h.is_valid())
+
+        h2 = s.find_torrent(lt.sha1_hash(b"a" * 20))
+        self.assertTrue(h2.is_valid())
+        h3 = s.find_torrent(lt.sha1_hash(b"b" * 20))
+        self.assertFalse(h3.is_valid())
+
+        self.assertEqual(h, h2)
+        self.assertNotEqual(h, h3)
+
+    def test_add_torrent_info_hash(self):
+        s = lt.session(settings)
+        h = s.add_torrent({
+                           'info_hash': b'a' * 20,
+                           'info_hashes': b'a' * 32,
+                           'save_path': '.'})
+
+        time.sleep(1)
+        alerts = s.pop_alerts()
+
+        while len(alerts) > 0:
+            a = alerts.pop(0)
+            print(a)
+
+        self.assertTrue(h.is_valid())
+        self.assertEqual(h.status().info_hashes, lt.info_hash_t(lt.sha256_hash(b'a' * 32)))
+
     def test_session_status(self):
         if not has_deprecated():
             return
@@ -776,8 +926,7 @@ class test_session(unittest.TestCase):
         self.assertEqual(s.get_settings()['user_agent'], 'test123')
 
     def test_post_session_stats(self):
-        s = lt.session({'alert_mask': lt.alert.category_t.stats_notification,
-                        'enable_dht': False})
+        s = lt.session({'alert_mask': 0, 'enable_dht': False})
         s.post_session_stats()
         alerts = []
         # first the stats headers log line. but not if logging is disabled
@@ -802,13 +951,17 @@ class test_session(unittest.TestCase):
         self.assertTrue(len(a.values) > 0)
 
     def test_post_dht_stats(self):
-        s = lt.session({'alert_mask': lt.alert.category_t.stats_notification, 'enable_dht': False})
+        s = lt.session({'alert_mask': 0, 'enable_dht': False})
         s.post_dht_stats()
         alerts = []
-        # first the stats headers log line. but not if logging is disabled
+        cnt = 0
         while len(alerts) == 0:
             s.wait_for_alert(1000)
             alerts = s.pop_alerts()
+            cnt += 1
+            if cnt > 60:
+                print('no dht_stats_alert in 1 minute!')
+                sys.exit(1)
         a = alerts.pop(0)
         self.assertTrue(isinstance(a, lt.dht_stats_alert))
         self.assertTrue(isinstance(a.active_requests, list))
@@ -975,6 +1128,11 @@ class test_peer_info(unittest.TestCase):
 
 class test_dht_settings(unittest.TestCase):
 
+    def test_dht_get_peers(self):
+        session = lt.session();
+        info_hash = lt.sha1_hash(b"a" * 20)
+        session.dht_get_peers(info_hash);
+
     def test_construct(self):
 
         ds = lt.dht_settings()
