@@ -147,6 +147,14 @@ namespace aux {
 			return { static_cast<byte*>(m_mapping), static_cast<std::ptrdiff_t>(m_size) };
 		}
 
+		// hint the kernel that we probably won't need this part of the file
+		// anytime soon
+		void dont_need(span<byte const> range);
+
+		// hint the kernel that the given (dirty) range of pages should be
+		// flushed to disk
+		void page_out(span<byte const> range);
+
 		std::int64_t m_size;
 #if TORRENT_HAVE_MAP_VIEW_OF_FILE
 		file_mapping_handle m_file;
@@ -175,6 +183,19 @@ namespace aux {
 			return m_mapping->memory();
 		}
 
+		void dont_need(span<byte const> range)
+		{
+			TORRENT_ASSERT(m_mapping);
+			m_mapping->dont_need(range);
+		}
+
+		void page_out(span<byte const> range)
+		{
+			TORRENT_ASSERT(m_mapping);
+			m_mapping->page_out(range);
+		}
+
+
 	private:
 		explicit file_view(std::shared_ptr<file_mapping> m) : m_mapping(std::move(m)) {}
 		std::shared_ptr<file_mapping> m_mapping;
