@@ -1,6 +1,8 @@
 /*
 
-Copyright (c) 2013-2016, Arvid Norberg
+Copyright (c) 2013-2015, 2017, 2019, Arvid Norberg
+Copyright (c) 2016, Steven Siloti
+Copyright (c) 2016, Alden Torres
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
@@ -37,9 +39,10 @@ POSSIBILITY OF SUCH DAMAGE.
 #include "libtorrent/address.hpp"
 #include "libtorrent/bloom_filter.hpp"
 #include "libtorrent/time.hpp" // for time_point
+#include "libtorrent/aux_/session_interface.hpp" // for ip_source_t
+
+namespace libtorrent {
 
-namespace libtorrent
-{
 	// this is an object that keeps the state for a single external IP
 	// based on peoples votes
 	struct TORRENT_EXTRA_EXPORT ip_voter
@@ -48,7 +51,7 @@ namespace libtorrent
 
 		// returns true if a different IP is the top vote now
 		// i.e. we changed our idea of what our external IP is
-		bool cast_vote(address const& ip, int source_type, address const& source);
+		bool cast_vote(address const& ip, aux::ip_source_t source_type, address const& source);
 
 		address external_address() const { return m_external_address; }
 
@@ -58,16 +61,14 @@ namespace libtorrent
 
 		struct external_ip_t
 		{
-			external_ip_t(): sources(0), num_votes(0) {}
+			bool add_vote(sha1_hash const& k, aux::ip_source_t type);
 
-			bool add_vote(sha1_hash const& k, int type);
-
-			// we want to sort decending
+			// we want to sort descending
 			bool operator<(external_ip_t const& rhs) const
 			{
 				if (num_votes > rhs.num_votes) return true;
 				if (num_votes < rhs.num_votes) return false;
-				return sources > rhs.sources;
+				return static_cast<std::uint8_t>(sources) > static_cast<std::uint8_t>(rhs.sources);
 			}
 
 			// this is a bloom filter of the IPs that have
@@ -76,9 +77,9 @@ namespace libtorrent
 			// this is the actual external address
 			address addr;
 			// a bitmask of sources the reporters have come from
-			boost::uint16_t sources;
+			aux::ip_source_t sources{};
 			// the total number of votes for this IP
-			boost::uint16_t num_votes;
+			std::uint16_t num_votes = 0;
 		};
 
 		// this is a bloom filter of all the IPs that have
@@ -106,26 +107,30 @@ namespace libtorrent
 		time_point m_last_rotate;
 	};
 
-	// this keeps track of multiple external IPs (for now, just IPv6 and IPv4, but
-	// it could be extended to deal with loopback and local network addresses as well)
+	// stores one address for each combination of local/global and ipv4/ipv6
+	// use of this class should be avoided, get the IP from the appropriate
+	// listen interface wherever possible
 	struct TORRENT_EXTRA_EXPORT external_ip
 	{
-		// returns true if a different IP is the top vote now
-		// i.e. we changed our idea of what our external IP is
-		bool cast_vote(address const& ip, int source_type, address const& source);
+		external_ip()
+			: m_addresses{{address_v4(), address_v6()}, {address_v4(), address_v6()}}
+		{}
+
+		external_ip(address const& local4, address const& global4
+			, address const& local6, address const& global6);
 
 		// the external IP as it would be observed from `ip`
 		address external_address(address const& ip) const;
 
 	private:
 
-		// for now, assume one external IPv4 and one external IPv6 address
-		// 0 = IPv4 1 = IPv6
-		// TODO: 1 instead, have one instance per possible subnet, global IPv4, global IPv6, loopback, 192.168.x.x, 10.x.x.x, etc.
-		ip_voter m_vote_group[2];
+		// support one local and one global address per address family
+		// [0][n] = global [1][n] = local
+		// [n][0] = IPv4 [n][1] = IPv6
+		// TODO: 1 have one instance per possible subnet, 192.168.x.x, 10.x.x.x, etc.
+		address m_addresses[2][2];
 	};
 
 }
 
 #endif
-
