@@ -75,6 +75,7 @@ namespace libtorrent {
 	constexpr create_flags_t create_torrent::v2_only;
 	constexpr create_flags_t create_torrent::v1_only;
 	constexpr create_flags_t create_torrent::canonical_files;
+	constexpr create_flags_t create_torrent::no_attributes;
 
 namespace {
 
@@ -131,7 +132,9 @@ namespace {
 		else
 		{
 			// #error use the fields from s
-			file_flags_t const file_flags = aux::get_file_attributes(f);
+			file_flags_t const file_flags = (flags & create_torrent::no_attributes)
+				? file_flags_t{}
+				: aux::get_file_attributes(f);
 
 			// mask all bits to check if the file is a symlink
 			if ((file_flags & file_storage::flag_symlink)
@@ -182,6 +185,7 @@ namespace {
 
 				auto const file_piece_offset = piece - file_first_piece;
 				auto const file_size = st->ct.files().file_size(current_file);
+				TORRENT_ASSERT(file_size > 0);
 				auto const file_blocks = st->ct.files().file_num_blocks(current_file);
 				auto const piece_blocks = st->ct.files().blocks_in_piece2(piece);
 				int const num_leafs = merkle_num_leafs(file_blocks);
@@ -449,15 +453,15 @@ namespace {
 			aux::throw_ex<system_error>(errors::invalid_piece_size);
 		}
 
-		m_files.set_piece_length(piece_size);
+		fs.set_piece_length(piece_size);
 		if (!(flags & v1_only) || (flags & canonical_files))
-			m_files.canonicalize();
-		m_files.set_num_pieces(aux::calc_num_pieces(m_files));
-		TORRENT_ASSERT(m_files.piece_length() > 0);
+			fs.canonicalize();
+		fs.set_num_pieces(aux::calc_num_pieces(fs));
+		TORRENT_ASSERT(fs.piece_length() > 0);
 	}
 
 	create_torrent::create_torrent(torrent_info const& ti)
-		: m_files(const_cast<file_storage&>(ti.files()))
+		: m_files(ti.files())
 		, m_creation_date(::time(nullptr))
 		, m_multifile(ti.num_files() > 1)
 		, m_private(ti.priv())
@@ -510,6 +514,7 @@ namespace {
 			{
 				// don't include merkle hash trees for pad files
 				if (m_files.pad_file_at(i)) continue;
+				if (m_files.file_size(i) == 0) continue;
 
 				auto const file_size = m_files.file_size(i);
 				if (file_size <= m_files.piece_length())
@@ -935,6 +940,7 @@ namespace {
 		TORRENT_ASSERT_PRECOND(piece < piece_index_t::diff_type(m_files.file_num_pieces(file)));
 		TORRENT_ASSERT_PRECOND(!m_files.pad_file_at(file));
 		TORRENT_ASSERT_PRECOND(!h.is_all_zeros());
+		TORRENT_ASSERT_PRECOND(m_files.file_num_pieces(file) > 0);
 
 		if (m_v1_only)
 			aux::throw_ex<system_error>(errors::invalid_hash_entry);
