@@ -114,6 +114,7 @@ typedef struct _FILE_ALLOCATED_RANGE_BUFFER {
 		return (in.Length.QuadPart != out[0].Length.QuadPart);
 	}
 
+#ifndef TORRENT_WINRT
 	std::once_flag g_once_flag;
 
 	void acquire_manage_volume_privs()
@@ -161,6 +162,7 @@ typedef struct _FILE_ALLOCATED_RANGE_BUFFER {
 
 		AdjustTokenPrivileges(token, FALSE, &privs, 0, nullptr, nullptr);
 	}
+#endif // TORRENT_WINRT
 
 #endif // TORRENT_WINDOWS
 
@@ -599,6 +601,10 @@ file_mapping::file_mapping(file_handle file, open_mode_t const mode, std::int64_
 		// with large disk caches)
 		// ignore errors here, since this is best-effort
 			| MADV_DONTDUMP
+#endif
+#ifdef MADV_NOCORE
+		// This is the BSD counterpart to exclude a range from core dumps
+			| MADV_NOCORE
 #endif
 		;
 		if (advise != 0)
@@ -683,6 +689,52 @@ file_mapping::file_mapping(file_mapping&& rhs)
 		return file_view(shared_from_this());
 	}
 
+
+void file_mapping::dont_need(span<byte const> range)
+{
+	auto* const start = const_cast<byte*>(range.data());
+	auto const size = static_cast<std::size_t>(range.size());
+
+#if TORRENT_USE_MADVISE
+	int const advise = 0
+#if defined TORRENT_LINUX && defined MADV_COLD
+		| MADV_COLD
+#elif !defined TORRENT_LINUX && defined MADV_DONTNEED
+		// note that MADV_DONTNEED is broken on Linux. It can destroy data. We
+		// cannot use it
+		| MADV_DONTNEED
+#endif
+	;
+
+	if (advise)
+		::madvise(start, size, advise);
+#endif
+#ifndef TORRENT_WINDOWS
+	::msync(start, size, MS_INVALIDATE);
+#else
+	TORRENT_UNUSED(start);
+	TORRENT_UNUSED(size);
+#endif
+}
+
+void file_mapping::page_out(span<byte const> range)
+{
+#if TORRENT_HAVE_MAP_VIEW_OF_FILE
+	// ignore errors, this is best-effort
+	FlushViewOfFile(range.data(), static_cast<std::size_t>(range.size()));
+#else
+
+	auto* const start = const_cast<byte*>(range.data());
+	auto const size = static_cast<std::size_t>(range.size());
+#if TORRENT_USE_MADVISE && defined MADV_PAGEOUT
+	::madvise(start, size, MADV_PAGEOUT);
+#endif
+
+	::msync(start, size, MS_ASYNC);
+
+#endif // MAP_VIEW_OF_FILE
+}
+
 } // aux
 } // libtorrent
 
