@@ -52,10 +52,6 @@ POSSIBILITY OF SUCH DAMAGE.
 #include "libtorrent/aux_/disable_warnings_push.hpp"
 #include <boost/optional.hpp>
 
-#if TORRENT_HAS_SYMLINK
-#include <unistd.h> // for symlink()
-#endif
-
 #include "libtorrent/aux_/disable_warnings_pop.hpp"
 
 #include "libtorrent/mmap_storage.hpp"
@@ -72,6 +68,44 @@ POSSIBILITY OF SUCH DAMAGE.
 
 namespace libtorrent {
 
+namespace {
+
+error_code translate_error(std::system_error const& err, bool const write)
+{
+	// We don't really know why we failed to read or write. SIGBUS essentially
+	// means I/O failure. We assume that if we were writing, the failure was
+	// because of disk full
+	if (write)
+	{
+#ifdef TORRENT_WINDOWS
+		if (err.code() == std::error_code(sig::seh_errors::in_page_error))
+			return error_code(boost::system::errc::no_space_on_device, generic_category());
+#else
+		if (err.code() == std::error_code(sig::errors::bus))
+			return error_code(boost::system::errc::no_space_on_device, generic_category());
+#endif
+	}
+
+#if BOOST_VERSION >= 107700
+
+#ifdef TORRENT_WINDOWS
+	if (err.code() == std::error_code(sig::seh_errors::in_page_error))
+		return error_code(boost::system::errc::io_error, generic_category());
+#else
+	if (err.code() == std::error_code(sig::errors::bus))
+		return error_code(boost::system::errc::io_error, generic_category());
+#endif
+
+	return err.code();
+#else
+
+	return error_code(boost::system::errc::io_error, generic_category());
+
+#endif
+}
+} // namespace
+
+
 	mmap_storage::mmap_storage(storage_params const& params
 		, aux::file_view_pool& pool)
 		: m_files(params.files)
@@ -139,21 +173,31 @@ namespace libtorrent {
 
 				if (m_part_file && use_partfile(i))
 				{
-					m_part_file->export_file([&f](std::int64_t file_offset, span<char> buf)
+					try
 					{
-						auto file_range = f->range().subspan(std::ptrdiff_t(file_offset));
-						TORRENT_ASSERT(file_range.size() >= buf.size());
-						sig::try_signal([&]{
-							std::memcpy(const_cast<char*>(file_range.data()), buf.data()
-								, static_cast<std::size_t>(buf.size()));
-							});
-					}, fs.file_offset(i), fs.file_size(i), ec.ec);
+						m_part_file->export_file([&f](std::int64_t file_offset, span<char> buf)
+							{
+							auto file_range = f->range().subspan(std::ptrdiff_t(file_offset));
+							TORRENT_ASSERT(file_range.size() >= buf.size());
+							sig::try_signal([&]{
+								std::memcpy(const_cast<char*>(file_range.data()), buf.data()
+									, static_cast<std::size_t>(buf.size()));
+								});
+							}, fs.file_offset(i), fs.file_size(i), ec.ec);
 
-					if (ec)
+						if (ec)
+						{
+							ec.file(i);
+							ec.operation = operation_t::partfile_write;
+							prio = m_file_priority;
+							return;
+						}
+					}
+					catch (std::system_error const& err)
 					{
 						ec.file(i);
 						ec.operation = operation_t::partfile_write;
-						prio = m_file_priority;
+						ec.ec = translate_error(err, true);
 						return;
 					}
 				}
@@ -165,7 +209,7 @@ namespace libtorrent {
 				// so we just don't use a partfile for this file
 
 				std::string const fp = fs.file_path(i, m_save_path);
-				if (exists(fp, ec.ec)) use_partfile(i, false);
+				bool const file_exists = exists(fp, ec.ec);
 				if (ec.ec)
 				{
 					ec.file(i);
@@ -173,6 +217,7 @@ namespace libtorrent {
 					prio = m_file_priority;
 					return;
 				}
+				use_partfile(i, !file_exists);
 /*
 				auto f = open_file(sett, i, aux::open_mode::read_only, ec);
 				if (ec.ec != boost::system::errc::no_such_file_or_directory)
@@ -231,11 +276,17 @@ namespace libtorrent {
 
 	void mmap_storage::use_partfile(file_index_t const index, bool const b)
 	{
-		if (index >= m_use_partfile.end_index()) m_use_partfile.resize(static_cast<int>(index) + 1, true);
+		if (index >= m_use_partfile.end_index())
+		{
+			// no need to extend this array if we're just setting it to "true",
+			// that's default already
+			if (b) return;
+			m_use_partfile.resize(static_cast<int>(index) + 1, true);
+		}
 		m_use_partfile[index] = b;
 	}
 
-	void mmap_storage::initialize(settings_interface const& sett, storage_error& ec)
+	status_t mmap_storage::initialize(settings_interface const& sett, storage_error& ec)
 	{
 		m_stat_cache.reserve(files().num_files());
 
@@ -253,6 +304,7 @@ namespace libtorrent {
 		}
 
 		file_storage const& fs = files();
+		status_t ret{};
 		// if some files have priority 0, we need to check if they exist on the
 		// filesystem, in which case we won't use a partfile for them.
 		// this is to be backwards compatible with previous versions of
@@ -262,112 +314,34 @@ namespace libtorrent {
 			if (m_file_priority[i] != dont_download || fs.pad_file_at(i))
 				continue;
 
-			file_status s;
-			std::string const file_path = fs.file_path(i, m_save_path);
 			error_code err;
-			stat_file(file_path, &s, err);
-			if (!err)
+			auto const size = m_stat_cache.get_filesize(i, fs, m_save_path, err);
+			if (!err && size > 0)
 			{
 				use_partfile(i, false);
+				if (size > fs.file_size(i))
+					ret = ret | status_t::oversized_file;
 			}
 			else
 			{
+				// we may have earlier determined we *can't* use a partfile for
+				// this file, we need to be able to change our mind in case the
+				// file disappeared
+				use_partfile(i, true);
 				need_partfile();
 			}
 		}
-		// first, create zero-sized files
-		for (auto const file_index : fs.file_range())
-		{
-			// ignore files that have priority 0
-			if (m_file_priority.end_index() > file_index
-				&& m_file_priority[file_index] == dont_download)
-			{
-				continue;
-			}
 
-			// ignore pad files
-			if (fs.pad_file_at(file_index)) continue;
-
-			// this is just to see if the file exists
-			error_code err;
-			m_stat_cache.get_filesize(file_index, fs, m_save_path, err);
-
-			if (err && err != boost::system::errc::no_such_file_or_directory)
-			{
-				ec.file(file_index);
-				ec.operation = operation_t::file_stat;
-				ec.ec = err;
-				break;
-			}
-
-			// if the file is empty and doesn't already exist, create it
-			// deliberately don't truncate files that already exist
-			// if a file is supposed to have size 0, but already exists, we will
-			// never truncate it to 0.
-			if (fs.file_size(file_index) == 0)
-			{
-#if TORRENT_HAS_SYMLINK
-				// create symlinks
-				if (fs.file_flags(file_index) & file_storage::flag_symlink)
-				{
-					std::string path = fs.file_path(file_index, m_save_path);
-					create_directories(parent_path(path), ec.ec);
-					if (ec)
-					{
-						ec.ec = error_code(errno, generic_category());
-						ec.file(file_index);
-						ec.operation = operation_t::mkdir;
-						break;
-					}
-					// we make the symlink target relative to the link itself
-					std::string const target = lexically_relative(
-						parent_path(fs.file_path(file_index)), fs.symlink(file_index));
-					std::string const link = fs.file_path(file_index, m_save_path);
-					if (::symlink(target.c_str(), link.c_str()) != 0)
-					{
-						int const error = errno;
-						if (error == EEXIST)
-						{
-							// if the file exist, it may be a symlink already. if so,
-							// just verify the link target is what it's supposed to be
-							// note that readlink() does not null terminate the buffer
-							char buffer[512];
-							auto const ret = ::readlink(link.c_str(), buffer, sizeof(buffer));
-							if (ret <= 0 || target != string_view(buffer, std::size_t(ret)))
-							{
-								ec.ec = error_code(error, generic_category());
-								ec.file(file_index);
-								ec.operation = operation_t::symlink;
-								return;
-							}
-						}
-						else
-						{
-							ec.ec = error_code(error, generic_category());
-							ec.file(file_index);
-							ec.operation = operation_t::symlink;
-							return;
-						}
-					}
-				}
-				else
-#endif
-				if (err == boost::system::errc::no_such_file_or_directory)
-				{
-					// just creating the file is enough to make it zero-sized. If
-					// there's a race here and some other process truncates the file,
-					// it's not a problem, we won't access empty files ever again
-					ec.ec.clear();
-					auto f = open_file(sett, file_index, aux::open_mode::write
-						| aux::open_mode::random_access | aux::open_mode::truncate, ec);
-					if (ec) return;
-				}
-			}
-			ec.ec.clear();
-		}
+		aux::initialize_storage(fs, m_save_path, m_stat_cache, m_file_priority
+			, [&sett, this](file_index_t const file_index, storage_error& e)
+			{ open_file(sett, file_index, aux::open_mode::write, e); }
+			, aux::create_symlink
+			, [&ret](file_index_t, std::int64_t) { ret = ret | status_t::oversized_file; }
+			, ec);
 
 		// close files that were opened in write mode
 		m_pool.release(storage_index());
+		return ret;
 	}
 
 	bool mmap_storage::has_any_file(storage_error& ec)
@@ -384,14 +358,13 @@ namespace libtorrent {
 		if (!ec) return true;
 
 		// the part file not existing is expected
-		if (ec && ec.ec == boost::system::errc::no_such_file_or_directory)
+		if (ec.ec == boost::system::errc::no_such_file_or_directory)
 			ec.ec.clear();
 
 		if (ec)
 		{
 			ec.file(torrent_status::error_file_partfile);
 			ec.operation = operation_t::file_stat;
-			return false;
 		}
 		return false;
 	}
@@ -527,13 +500,15 @@ namespace libtorrent {
 	int mmap_storage::readv(settings_interface const& sett
 		, span<iovec_t const> bufs
 		, piece_index_t const piece, int const offset
-		, aux::open_mode_t const flags, storage_error& error)
+		, aux::open_mode_t const mode
+		, disk_job_flags_t const flags
+		, storage_error& error)
 	{
 #ifdef TORRENT_SIMULATE_SLOW_READ
 		std::this_thread::sleep_for(seconds(1));
 #endif
 		return readwritev(files(), bufs, piece, offset, error
-			, [this, flags, &sett](file_index_t const file_index
+			, [this, mode, flags, &sett](file_index_t const file_index
 				, std::int64_t const file_offset
 				, span<iovec_t const> vec, storage_error& ec)
 		{
@@ -560,34 +535,48 @@ namespace libtorrent {
 				return ret;
 			}
 
-			auto handle = open_file(sett, file_index, flags, ec);
+			auto handle = open_file(sett, file_index, mode, ec);
 			if (ec) return -1;
 
 			int ret = 0;
 			error_code e;
 			span<byte const> file_range = handle->range();
-			if (file_range.size() > file_offset)
-			{
-				file_range = file_range.subspan(static_cast<std::ptrdiff_t>(file_offset));
-				for (auto buf : vec)
-				{
-					if (file_range.empty()) break;
-					if (file_range.size() < buf.size()) buf = buf.first(file_range.size());
-
-					sig::try_signal([&]{
-						std::memcpy(buf.data(), const_cast<char*>(file_range.data())
-							, static_cast<std::size_t>(buf.size()));
-					});
-
-					file_range = file_range.subspan(buf.size());
-					ret += static_cast<int>(buf.size());
-				}
-			}
 
 			// set this unconditionally in case the upper layer would like to treat
 			// short reads as errors
 			ec.operation = operation_t::file_read;
 
+			try
+			{
+				if (file_range.size() > file_offset)
+				{
+					file_range = file_range.subspan(static_cast<std::ptrdiff_t>(file_offset));
+					for (auto buf : vec)
+					{
+						if (file_range.empty()) break;
+						if (file_range.size() < buf.size()) buf = buf.first(file_range.size());
+
+						sig::try_signal([&]{
+							std::memcpy(buf.data(), const_cast<char*>(file_range.data())
+								, static_cast<std::size_t>(buf.size()));
+							});
+
+						file_range = file_range.subspan(buf.size());
+						ret += static_cast<int>(buf.size());
+					}
+					if (flags & disk_interface::volatile_read)
+						handle->dont_need(file_range);
+					if (flags & disk_interface::flush_piece)
+						handle->page_out(file_range);
+				}
+			}
+			catch (std::system_error const& err)
+			{
+				ec.file(file_index);
+				ec.ec = translate_error(err, false);
+				return -1;
+			}
+
 			// we either get an error or 0 or more bytes read
 			TORRENT_ASSERT(e || ret > 0);
 			TORRENT_ASSERT(ret <= bufs_size(vec));
@@ -606,10 +595,12 @@ namespace libtorrent {
 	int mmap_storage::writev(settings_interface const& sett
 		, span<iovec_t const> bufs
 		, piece_index_t const piece, int const offset
-		, aux::open_mode_t const flags, storage_error& error)
+		, aux::open_mode_t const mode
+		, disk_job_flags_t const flags
+		, storage_error& error)
 	{
 		return readwritev(files(), bufs, piece, offset, error
-			, [this, flags, &sett](file_index_t const file_index
+			, [this, mode, flags, &sett](file_index_t const file_index
 				, std::int64_t const file_offset
 				, span<iovec_t const> vec, storage_error& ec)
 		{
@@ -645,32 +636,47 @@ namespace libtorrent {
 			m_stat_cache.set_dirty(file_index);
 
 			auto handle = open_file(sett, file_index
-				, aux::open_mode::write | flags, ec);
+				, aux::open_mode::write | mode, ec);
 			if (ec) return -1;
 
 			int ret = 0;
 			error_code e;
 			span<byte> file_range = handle->range().subspan(static_cast<std::ptrdiff_t>(file_offset));
-			for (auto buf : vec)
+
+			// set this unconditionally in case the upper layer would like to treat
+			// short reads as errors
+			ec.operation = operation_t::file_write;
+
+			try
 			{
-				TORRENT_ASSERT(file_range.size() >= buf.size());
+				for (auto buf : vec)
+				{
+					TORRENT_ASSERT(file_range.size() >= buf.size());
 
-				sig::try_signal([&]{
-					std::memcpy(const_cast<char*>(file_range.data()), buf.data(), static_cast<std::size_t>(buf.size()));
-				});
+					sig::try_signal([&]{
+						std::memcpy(const_cast<char*>(file_range.data()), buf.data(), static_cast<std::size_t>(buf.size()));
+						});
 
-				file_range = file_range.subspan(buf.size());
-				ret += static_cast<int>(buf.size());
+					file_range = file_range.subspan(buf.size());
+					ret += static_cast<int>(buf.size());
+				}
+
+				if (flags & disk_interface::volatile_read)
+					handle->dont_need(file_range);
+				if (flags & disk_interface::flush_piece)
+					handle->page_out(file_range);
+			}
+			catch (std::system_error const& err)
+			{
+				ec.file(file_index);
+				ec.ec = translate_error(err, true);
+				return -1;
 			}
 
 #if TORRENT_HAVE_MAP_VIEW_OF_FILE
 			m_pool.record_file_write(storage_index(), file_index, ret);
 #endif
 
-			// set this unconditionally in case the upper layer would like to treat
-			// short reads as errors
-			ec.operation = operation_t::file_write;
-
 			if (e)
 			{
 				ec.ec = e;
@@ -685,7 +691,9 @@ namespace libtorrent {
 	int mmap_storage::hashv(settings_interface const& sett
 		, hasher& ph, std::ptrdiff_t const len
 		, piece_index_t const piece, int const offset
-		, aux::open_mode_t const flags, storage_error& error)
+		, aux::open_mode_t const mode
+		, disk_job_flags_t const flags
+		, storage_error& error)
 	{
 #ifdef TORRENT_SIMULATE_SLOW_READ
 		std::this_thread::sleep_for(seconds(1));
@@ -695,7 +703,7 @@ namespace libtorrent {
 		span<iovec_t> dummy2(&dummy1, 1);
 
 		return readwritev(files(), dummy2, piece, offset, error
-			, [this, flags, &ph, &sett](file_index_t const file_index
+			, [this, mode, flags, &ph, &sett](file_index_t const file_index
 				, std::int64_t const file_offset
 				, span<iovec_t const> vec, storage_error& ec)
 		{
@@ -732,7 +740,7 @@ namespace libtorrent {
 				return ret;
 			}
 
-			auto handle = open_file(sett, file_index, flags, ec);
+			auto handle = open_file(sett, file_index, mode, ec);
 			if (ec) return -1;
 
 			int ret = 0;
@@ -746,6 +754,10 @@ namespace libtorrent {
 					ph.update({const_cast<char const*>(file_range.data()), file_range.size()});
 				});
 				ret += static_cast<int>(file_range.size());
+				if (flags & disk_interface::volatile_read)
+					handle->dont_need(file_range);
+				if (flags & disk_interface::flush_piece)
+					handle->page_out(file_range);
 			}
 
 			return ret;
@@ -755,7 +767,9 @@ namespace libtorrent {
 	int mmap_storage::hashv2(settings_interface const& sett
 		, hasher256& ph, std::ptrdiff_t const len
 		, piece_index_t const piece, int const offset
-		, aux::open_mode_t const flags, storage_error& error)
+		, aux::open_mode_t const mode
+		, disk_job_flags_t const flags
+		, storage_error& error)
 	{
 		std::int64_t const start_offset = static_cast<int>(piece) * std::int64_t(files().piece_length()) + offset;
 		file_index_t const file_index = files().file_index_at_offset(start_offset);
@@ -782,7 +796,7 @@ namespace libtorrent {
 			return ret;
 		}
 
-		auto handle = open_file(sett, file_index, flags, error);
+		auto handle = open_file(sett, file_index, mode, error);
 		if (error) return -1;
 
 		span<byte const> file_range = handle->range();
@@ -791,6 +805,10 @@ namespace libtorrent {
 		file_range = file_range.subspan(std::ptrdiff_t(file_offset));
 		file_range = file_range.first(std::min(std::ptrdiff_t(len), file_range.size()));
 		ph.update(file_range);
+		if (flags & disk_interface::volatile_read)
+			handle->dont_need(file_range);
+		if (flags & disk_interface::flush_piece)
+			handle->page_out(file_range);
 
 		return static_cast<int>(file_range.size());
 	}
@@ -828,7 +846,14 @@ namespace libtorrent {
 
 		boost::optional<aux::file_view> h = open_file_impl(sett, file, mode, ec);
 		if ((mode & aux::open_mode::write)
-			&& ec.ec == boost::system::errc::no_such_file_or_directory)
+			&& (ec.ec == boost::system::errc::no_such_file_or_directory
+#ifdef TORRENT_WINDOWS
+				// this is a workaround for improper handling of files on windows shared drives.
+				// if the directory on a shared drive does not exist,
+				// windows returns ERROR_IO_DEVICE instead of ERROR_FILE_NOT_FOUND
+				|| ec.ec == error_code(ERROR_IO_DEVICE, system_category())
+#endif
+		))
 		{
 			// this means the directory the file is in doesn't exist.
 			// so create it
@@ -850,7 +875,6 @@ namespace libtorrent {
 		if (ec.ec)
 		{
 			ec.file(file);
-			ec.operation = operation_t::file_open;
 			return {};
 		}
 		TORRENT_ASSERT(h);
@@ -886,8 +910,9 @@ namespace libtorrent {
 		}
 
 		// if we have a cache already, don't store the data twice by leaving it in the OS cache as well
-		if (sett.get_int(settings_pack::disk_io_write_mode)
-			== settings_pack::disable_os_cache)
+		auto const write_mode = sett.get_int(settings_pack::disk_io_write_mode);
+		if (write_mode == settings_pack::disable_os_cache
+			|| write_mode == settings_pack::write_through)
 		{
 			mode |= aux::open_mode::no_cache;
 		}
