@@ -42,6 +42,7 @@ POSSIBILITY OF SUCH DAMAGE.
 #include "libtorrent/disk_interface.hpp"
 #include "libtorrent/mmap_disk_io.hpp"
 #include "libtorrent/posix_disk_io.hpp"
+#include "libtorrent/platform_util.hpp"
 
 namespace libtorrent {
 
@@ -198,9 +199,6 @@ namespace {
 		// unchoke all peers
 		set.set_int(settings_pack::unchoke_slots_limit, -1);
 
-		set.set_int(settings_pack::read_cache_line_size, 32);
-		set.set_int(settings_pack::write_cache_line_size, 256);
-
 		// the max number of bytes pending write before we throttle
 		// download rate
 		set.set_int(settings_pack::max_queued_disk_bytes, 7 * 1024 * 1024);
@@ -340,8 +338,11 @@ namespace {
 		{
 			// start a thread for the message pump
 			auto s = m_io_service;
-			m_thread = std::make_shared<std::thread>(
-				[=] { s->run(); });
+			m_thread = std::make_shared<std::thread>([=]
+			{
+				set_thread_name("Network");
+				s->run();
+			});
 		}
 	}
 
