@@ -1,6 +1,8 @@
 /*
 
-Copyright (c) 2008, Arvid Norberg
+Copyright (c) 2005, 2008, 2010, 2012-2020, Arvid Norberg
+Copyright (c) 2017, Andrei Kurushin
+Copyright (c) 2018, Steven Siloti
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
@@ -73,7 +75,11 @@ namespace {
 // out, such as the log
 int old_stdout = -1;
 int old_stderr = -1;
+#ifdef _WIN32
+bool redirect_stdout = !IsDebuggerPresent();
+#else
 bool redirect_stdout = true;
+#endif
 // sanitizer output will go to stderr and we won't get an opportunity to print
 // it, so don't redirect stderr by default
 bool redirect_stderr = false;
@@ -332,9 +338,9 @@ int EXPORT main(int argc, char const* argv[])
 		if (argv[0] == "-l"_sv || argv[0] == "--list"_sv)
 		{
 			std::printf("TESTS:\n");
-			for (int i = 0; i < unit_test::g_num_unit_tests; ++i)
+			for (int i = 0; i < ::unit_test::g_num_unit_tests; ++i)
 			{
-				std::printf(" - %s\n", unit_test::g_unit_tests[i].name);
+				std::printf(" - %s\n", ::unit_test::g_unit_tests[i].name);
 			}
 			return 0;
 		}
@@ -449,7 +455,7 @@ int EXPORT main(int argc, char const* argv[])
 			return 1;
 		}
 
-		auto& t = unit_test::g_unit_tests[i];
+		auto& t = ::unit_test::g_unit_tests[i];
 
 		if (redirect_stdout || redirect_stderr)
 		{
@@ -502,7 +508,7 @@ int EXPORT main(int argc, char const* argv[])
 		setbuf(stdout, nullptr);
 		setbuf(stderr, nullptr);
 
-		unit_test::g_test_idx = i;
+		::unit_test::g_test_idx = i;
 		current_test = &t;
 
 		std::printf("cwd: %s\n", unit_dir.c_str());
@@ -516,11 +522,10 @@ int EXPORT main(int argc, char const* argv[])
 		{
 #endif
 
-#if defined TORRENT_BUILD_SIMULATOR
+			std::srand(unsigned(std::hash<std::string>{}(executable)) + unsigned(i));
 			lt::aux::random_engine().seed(0x82daf973);
-#endif
 
-			unit_test::g_test_failures = 0;
+			::unit_test::g_test_failures = 0;
 			(*t.fun)();
 #ifndef BOOST_NO_EXCEPTIONS
 		}
@@ -547,12 +552,12 @@ int EXPORT main(int argc, char const* argv[])
 
 		if (!tests_to_run.empty()) tests_to_run.erase(t.name);
 
-		if (unit_test::g_test_failures > 0)
+		if (::unit_test::g_test_failures > 0)
 		{
 			output_test_log_to_terminal();
 		}
 
-		t.num_failures = unit_test::g_test_failures;
+		t.num_failures = ::unit_test::g_test_failures;
 		t.run = true;
 		++num_run;
 
